def is_small(im1, im2, header1, header2):
    """ 
        Fuction determines which image has the least amount of pixels
        And, initiates necessary parameters for the resize function

        Args: 
            im1 (numpy.ndarray): data for first image
            im2 (numpy.ndarray): data for second image
            header1 (astropy.io.fits.header.Header): fits file header for first image
            header2 (astropy.io.fits.header.Header): fits file header for second image

        Returns: 
            small_im (numpy.ndarray): smallest image data, 
            deltapix1 (float): pixel thickness (deg/pix) of smallest image,
            smallhead (astropy.io.fits.header.Header): header file of smallest image,
            big_im (numpy.ndarray): largest image data,
            new_dim (int): largest image length along x-direction in pixels,
            deltapix2 (float): pixel thickness (deg/pix) of largest image,
            bighead (astropy.io.fits.header.Header): header file of largest image
    """
    # NEED TO RESOLVE CASE FOR SIZE1 == SIZE2

    size1 = im1.shape[0]*im1.shape[1]
    size2 = im2.shape[0]*im2.shape[1]

    if size1 < size2:
        small_im = im1
        deltapix1 = header1['CDELT2']
        smallhead = header1
        big_im = im2
        deltapix2 = header2['CDELT2']
        bighead = header2
        new_dim = header2['NAXIS1']
    else:
        small_im = im2
        deltapix1 = header2['CDELT2']
        smallhead = header2
        big_im = im1
        deltapix2 = header1['CDELT2']
        bighead = header1
        new_dim = header1['NAXIS1']

    return small_im, deltapix1, smallhead, big_im, new_dim, deltapix2, bighead



from astropy.io import fits

def upload_fits(file):   
    """ 
        Fuction uploads a fits file

        Arg: 
            file (str): path to fits file with desired image

        Returns: 
            header (astropy.io.fits.header.Header): header file
            sim_data (numpy.ndarray): image data

    """
    with fits.open(file) as hdul:
        sim_data = hdul[0].data[0,0,:,:]
        header = hdul[0].header
    
    return header, sim_data



from astropy.visualization import (AsinhStretch, ImageNormalize)
import matplotlib.pyplot as plt
import numpy as np

def make_image(data):
    """
        Function makes image

        Arg: 
            data (numpy.ndarray): image data

        Outputs: 
            plot of image
    """
    # Creates image for simulated observation at 0.87 mm 
    fig = plt.figure(figsize = (8,8))
    vmax = np.percentile(data, 99)
    vmin = np.percentile(data, 1)
    norm = ImageNormalize(vmin=vmin, vmax=vmax, stretch=AsinhStretch())
    snu = np.squeeze(data)

    im = plt.imshow(snu, origin='lower', cmap='inferno', norm=norm)



import numpy as np

def jyperbeam_to_jyperpix(header, data):
    """ 
        Fuction converts Jy/beam to Jy/pix

        Arg: 
            header (astropy.io.fits.header.Header): fits file header
            data (numpy.ndarray): image data

        Returns: 
            Jyperpix_data (numpy.ndarray): image data in Jy/pix

    """
    convert = np.pi/180 # deg to rads
    fwhm_to_sigma = 1./(8*np.log(2))**0.5

    bmaj = header['BMAJ']*convert
    bmin = header['BMIN']*convert

    beam_area = 2.*np.pi*(bmaj*bmin*fwhm_to_sigma*2) # in staradians
    Jyperpix_data = data/beam_area

    return Jyperpix_data



import numpy as np
from scipy.interpolate import interp2d

def resize_smalltolarge(im1, deltapix1, new_dim, deltapix2):   
    """
        Function resizes a small image to match the dimesions of a larger image with a different pixel size

        Args:
        im1 (numpy.ndarray): data for smaller image
        deltapix1 (float): pixel thickness of smaller image (deg/pix)
        new_dim (int): number of pixels along x-axis of larger image
        deltapix2 (float): pixel thickness of larger image (deg/pix)
    
        Returns: new_im (numpy.ndarray): resized image data
    """
    # Works if number of pixels in image1 is less than the number of pixels in image2
    
    # Creates a temporary matrix of the same length and width as image 2
    # But, with the same pixel size as image1
    # New pixels in the temporary matrix are filled in with zeros
    # The rest of the matrix contains image1
    
    # Interpolates temporary matrix to get new image with image2 dims (number of pixels & pixel size)
    
    # returns resized image

    im1_dim = len(im1) # number of pixels from image1
    
    if im1_dim < new_dim: # checks that number of pixels in image1 < number of pixels in image2
        
        # Creates temporary matrix
        diff = int((new_dim - im1_dim)/2.) # computes pixel difference between images and divides it by 2
        
        pad_with = ((diff,diff), (diff, diff)) # number of empty pixels to surround the matrix
        const_val = ((0,0), (0,0)) # values to add to the empty pixels
        
        # np.pad() takes image1, adds extra pixels and fills them with zeros
        temp_matrix = np.pad(im1, pad_with, 'constant', constant_values=const_val)
        
    # Creates the coordinates of the temporary matrix with pixel size: deltapix1
    temp_width = len(temp_matrix) # in pixels
    temp_halfwidth = temp_width/2.*deltapix1 # in deg
    
    xcoords = np.linspace(-temp_halfwidth, temp_halfwidth, temp_width)
    ycoords = np.linspace(-temp_halfwidth, temp_halfwidth, temp_width) # update for images of different length

    # Associates the coordinates to the temporary matrix
    temp_im = interp2d(xcoords, ycoords, temp_matrix)

    # Creates the coordinates of the resized image
    new_halfdim = new_dim/2.*deltapix2 # in deg
    
    new_xcoords = new_ycoords = np.linspace(-new_halfdim, new_halfdim, new_dim)

    # Interpolates the temporary matrix into the final resized image
    new_im = np.array(temp_im(new_xcoords, new_ycoords))
    
    return new_im



import numpy as np
from astrodesim.astrodesim import jyperbeam_to_jyperpix as convert
from astrodesim.astrodesim import spectral_index_conversion as make

def spectral_index_map(data1, data2, header1, header2):
    """ 
        Fuction creates image data in dust emission spectral index units from two images at different wavelengths

        Arg: 
            data1 (numpy.ndarray): data of first image
            data2 (numpy.ndarray): data of second image
            header1 (astropy.io.fits.header.Header): fits file header of first image
            header2 (astropy.io.fits.header.Header): fits file header of second image

        Returns: 
            im3 (numpy.ndarray): image data in dust emission spectral index units
    """
    # Converts intensity of image data from Jy/beam to Jy/pix
    im1 = convert.jyperbeam_to_jyperpix(header1, data1) 
    im2 = convert.jyperbeam_to_jyperpix(header2, data2) 

    c = 299792258 # speed of light in m/s

    lambda1 = c/header1['CRVAL4'] # frequency in Hz (need to test input for this)
    lambda2 = c/header2['CRVAL4'] # frequency in Hz 

    width = im1.shape[0]
    length = im2.shape[1]

    im3 = np.empty((width, length))

    for i in range(width):
        for j in range(length):
            flux1 = im1[i,j]
            flux2 = im2[i,j]
            im3[i,j] = make.spectral_index_conversion(flux1, flux2, lambda1, lambda2)

    return im3



from astrodesim.astrodesim import upload_fits as fits
from astrodesim.astrodesim import is_small as small
from astrodesim.astrodesim import resize_smalltolarge as resize
from astrodesim.astrodesim import spectral_index_map as alpha
from astrodesim.astrodesim import make_image as im

#class FeedMeFiles(object):
#    """
#    Class contains two fits files of the same image at different wavelengths

#    Args:
#        file1 (str): path to fits file with image at one wavelength
#        file2 (str): path to fits file with same image at another wavelength 
#    """
#    def __init__(self, file1, file2):
#        self.file1 = file1
#        self.file2 = file2

def map(file1, file2):
    """
    Function outputs a dust emission spectral index map

    Args: 
        files (FeedMeFiles): paths to file1 and file2
    """
    header1, sim_data1 = fits.upload_fits(file1) 
    header2, sim_data2 = fits.upload_fits(file2)

    smallim, deltapix1, new_header, bigim, new_dim, deltapix2, bigheader = small.is_small(sim_data1, sim_data2, header1, header2)

    new_im = resize.resize_smalltolarge(smallim, deltapix1, new_dim, deltapix2)

    desim = alpha.spectral_index_map(new_im, bigim, new_header, bigheader)

    im.make_image(desim)



import math

def spectral_index_conversion(flux1, flux2, wavelength1, wavelength2):
    """ 
        Fuction computes the dust emission spectral index from two intensity values

        Arg: 
            flux1 (float): Intensity (Jy/pix) of a single pixel from first image
            flux2 (float): Intensity (Jy/pix) of a single pixel from second image
            wavelength1 (float): wavelength of first image
            wavelength2 (float): wavelength of second image

        Returns: 
            alpha (float): dust emission spectral index
    """   
    flux_replacement = 1e-34
    
    # corrects for negative or zero values to be used in log calculations
    if flux1 < 0:
        flux1 = flux_replacement
        
    if flux2 <= 0:
        flux2 = flux_replacement

    flux_rat = math.log10(flux1/flux2)
    wavelength_rat = math.log10(wavelength2/wavelength1)
    alpha = flux_rat/wavelength_rat 

    return alpha
